/*
** Module   : tty.screen.js
** Abstract : screen management.
**
** Copyright (c) 2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**         Golden Code Development Corporation
**                 CONFIDENTIAL
**
** -#- -I- --Date-- ----------------------Description--------------------------
** 001 MAG 20140110 First version.
*/

tty.screen = (function()
{
   /* exports object */
   var my = {};
   /* canvas */
   var canvas;
   /* canvas 2d context */
   var ctx;
   /* canvas font */
   var font;
   /* screen dimension */
   var rows, cols;      
   /* font width and height */
   var dx, dy;

   /*
   ** Module   : video
   ** Abstract : screen video memory
   */
   var video = (function()
   {
      /* exports object */
      var my = {};
      /* video memory */
      var memo;      
      /* cell default values */
      
      /* initialize video memory */
      my.init = function(size, color)
      {
         memo = new Array(size.rows);
         for (i = 0;i < size.rows;i++) 
         {
            memo[i] = new Array(size.cols);
            for(j = 0;j < size.cols;j++)
            {
               cell = {'c':' ','f':'white','b':'black'};
               cell.f = color.f;
               cell.b = color.b;
               memo[i][j] = cell;
            }
         }
      };

      /* clear video memory */
      my.clear = function(rows, cols, font)
      {
         for (i = 0;i < rows;i++) 
         {
            for(j = 0;j < cols;j++)
            {
               cell = {'c':' ','f':'white','b':'black'};
               cell.f = color.f;
               cell.b = color.b;
               memo[i][j] = cell;
            }
         }
      };

      /* get cell value */
      my.get = function(row, col)
      {
         return memo[row][col];
      };

      /* set cell value*/
      my.put = function(row, col, cell)
      {
         memo[row][col] = cell;
      };

      /* set cell value*/
      my.putChar = function(c, row, col)
      {
         memo[row][col].c = c;
      };
      
      /* exports */
      return my;
   })();

   /*
   ** Module   : cursor 
   ** Abstract : screen cursor management
   */
   var cursor = (function(screen)
   {
      /* exports object */
      var my = {};
      /* visibility */
      var visible = true;

      /* hide cursor */
      my.hide = function()
      {
         visible = false;
         my.drawCursor(true);
      };
      
      /* show cursor */
      my.show = function()
      {
         visible = true;
         my.drawCursor(false);
      };

      /* advance cursor to next position */
      my.next = function(rows, cols)
      {
         my.col++;
         if (my.col === cols)
         {
            my.col = 0;
            my.row++;
            if (my.row === rows)
            {
               my.row = 0;
            }
         }
      };

      /* advance cursor to previous position */
      my.back = function(rows, cols)
      {
         my.col--;
         if (my.col < 0)
         {
            my.col = my.col + cols;
            my.row--;
            if (my.row < 0)
            {
               my.row = my.row + rows;
            }
         }
      };
      
      /* set cursor position */
      my.setPosition = function(row, col)
      {
         my.drawCursor(true);
         my.row = row;
         my.col = col;
         my.drawCursor(false);
      };
      
      /* draw cursor */
      my.drawCursor = function(off)
      {
         if (off)
            screen.printCharAt(my.row, my.col, false);
         else
            if (visible)
               screen.printCharAt(my.row, my.col, true);
      };

      /*
      ** Module   : blinking 
      ** Abstract : screen cursor blinking management
      */
      my.blinking = (function(cursor) 
      {
         /* exports object */
         var my = {};
         /* timer */
         var timer;

         /* stop blinking */
         my.off = function()
         {
            window.clearInterval(timer);
         };

         /* start blinking */
         my.on = function()
         {
            var blink = true;
            timer = window.setInterval(function()
            {
               cursor.drawCursor(blink);
               blink = !blink;
            }, 500);
         };

         /* exports */
         return my;
      }(my));
      
      /* initialize cursor */
      my.init = function(cursor)
      {
         my.row = 0;
         my.col = 0;
         my.type = cursor.type;
         visible = cursor.visible;
         if (cursor.blinking) 
            my.blinking.on();
      };
      
      /* exports */
      return my;
   })(my);
   
   /* print cell at (row, col) using cursor colours */
   my.printCharAt = function(row, col, inv)
   {
      var v = video.get(row, col);
      ctx.font = font;
      ctx.textBaseline = 'top';
      if (inv) 
      {
         ctx.fillStyle = v.f;
         if (cursor.type === 'solid')
         {
            ctx.fillRect(col * dx, row * dy, dx, dy);
            ctx.fillStyle = v.b;
         } 
         else 
         {
            ctx.fillRect(col * dx, ((row + 1) * dy) - 2, dx, 2);
            ctx.fillStyle = v.f;
         }
      } 
      else 
      {
         ctx.fillStyle = v.b;
         ctx.fillRect(col * dx, row * dy, dx, dy);
         ctx.fillStyle = v.f;
      }
      ctx.fillText(v.c, col * dx, row * dy);
   };

   /* print char at (row, col) */
   my.printChar = function(c, row, col)
   {
      var v = video.get(cursor.row, cursor.col);
      if (c != v.c)
      {
         video.putChar(c, row, col);
         my.printCharAt(row, col, false);
      }
   };
   
   /* print text at current cursor position */
   my.type = function(text)
   {
      cursor.hide();
      var c = cursor.col;
      var r = cursor.row;
      cursor.col=0;
      cursor.row=0;
      for(var i = 0;i < text.length;i++)
      {
         my.printChar(text.charAt(i), cursor.row, cursor.col);
         cursor.next(rows, cols);
      }
      cursor.col=c;
      cursor.row=r;
      cursor.show();
   };
   
   /* initialize canvas */
   my.init = function(cfg)
   {
      if (cfg.font.name === 'monospaced')
      {
         cfg.font.name = 'monospace'
      }
      rows = cfg.canvas.size.rows;
      cols = cfg.canvas.size.cols;    
      video.init(cfg.canvas.size, cfg.font.color);
      canvas = document.getElementById(cfg.canvas.id);
      ctx = canvas.getContext('2d');
      font = cfg.font.size + 'px ' + cfg.font.name;
      ctx.font = font;
      dx = ctx.measureText('M').width;
      dy = cfg.font.size;
      canvas.width = cols * dx;
      canvas.height = rows * dy;
      ctx.fillStyle = cfg.font.color.b;
      ctx.fillRect(0, 0, canvas.width, canvas.height);
      cursor.init(cfg.cursor);
   };
   
   /* cursor next */
   my.next = function()
   {
      cursor.next(rows, cols);
   };

   /* cursor back */
   my.back = function()
   {
      cursor.back(rows, cols);
   };

   /* set cursor position */   
   my.setCursorPosition = function(x, y)
   {
      cursor.setPosition(x, y);         
   }
   
   /* set cursor visibility */   
   my.setCursorStatus = function(on)
   {
      if (on)
         cursor.show();
      else
         cursor.hide();
   }

   /* set cursor visibility */   
   my.clear = function()
   {
      cursor.hide();
      video.clear(rows, cols, font);
      ctx.fillStyle = font.color.b;
      ctx.fillRect(0, 0, canvas.width, canvas.height);
      cursor.setPosition(0, 0);        
      cursor.show();
   }
   
   /* exports */
   return my;       
}());
